@@ -5,7 +5,7 @@
  *
  * KSBA is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * KSBA is distributed in the hope that it will be useful,
@@ -14,9 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -31,8 +29,10 @@
 #include "ber-help.h"
 #include "convert.h"
 #include "keyinfo.h"
+#include "sexp-parse.h"
 #include "cert.h"
 
+
 static const char oidstr_subjectKeyIdentifier[] = "2.5.29.14";
 static const char oidstr_keyUsage[]         = "2.5.29.15";
 static const char oidstr_subjectAltName[]   = "2.5.29.17";
@@ -119,8 +119,11 @@ ksba_cert_release (ksba_cert_t cert)
       xfree (cert->cache.extns);
     }
 
+  _ksba_asn_release_nodes (cert->root);
+  ksba_asn_tree_release (cert->asn_tree);
+
+  xfree (cert->image);
 
-  /* FIXME: release cert->root, ->asn_tree */
   xfree (cert);
 }
 
@@ -261,7 +264,10 @@ ksba_cert_read_der (ksba_cert_t cert, ksba_reader_t reader)
   if (cert->initialized)
     return gpg_error (GPG_ERR_CONFLICT); /* Fixme: should remove the old one */
 
-  /* fixme: clear old cert->root */
+  _ksba_asn_release_nodes (cert->root);
+  ksba_asn_tree_release (cert->asn_tree);
+  cert->root = NULL;
+  cert->asn_tree = NULL;
 
   err = ksba_asn_create_tree ("tmttv2", &cert->asn_tree);
   if (err)
@@ -343,7 +349,7 @@ ksba_cert_get_image (ksba_cert_t cert, size_t *r_length )
     {
       fprintf (stderr,"\nOops, ksba_cert_get_image failed: "
                "imagelen=%d  hdr=%d len=%d off=%d\n",
-               cert->imagelen, n->nhdr, n->len, n->off);
+               cert->imagelen, n->nhdr, (int)n->len, n->off);
       return NULL;
     }
   /*assert (n->nhdr + n->len + n->off <= cert->imagelen);*/
@@ -420,8 +426,10 @@ ksba_cert_hash (ksba_cert_t cert, int what,
 const char *
 ksba_cert_get_digest_algo (ksba_cert_t cert)
 {
+  gpg_error_t err;
   AsnNode n;
   char *algo;
+  size_t nread;
 
   if (!cert)
     {
@@ -437,11 +445,22 @@ ksba_cert_get_digest_algo (ksba_cert_t cert)
   if (cert->cache.digest_algo)
     return cert->cache.digest_algo;
   
-  n = _ksba_asn_find_node (cert->root,
-                           "Certificate.signatureAlgorithm.algorithm");
-  algo = _ksba_oid_node_to_str (cert->image, n);
-  if (!algo)
-    cert->last_error = gpg_error (GPG_ERR_UNKNOWN_ALGORITHM);
+/*   n = _ksba_asn_find_node (cert->root, */
+/*                            "Certificate.signatureAlgorithm.algorithm"); */
+/*   algo = _ksba_oid_node_to_str (cert->image, n); */
+/*   if (!algo) */
+/*     cert->last_error = gpg_error (GPG_ERR_UNKNOWN_ALGORITHM); */
+/*   else  */
+/*     cert->cache.digest_algo = algo; */
+
+  n = _ksba_asn_find_node (cert->root, "Certificate.signatureAlgorithm");
+  if (!n || n->off == -1)
+    err = gpg_error (GPG_ERR_UNKNOWN_ALGORITHM);
+  else
+    err = _ksba_parse_algorithm_identifier (cert->image + n->off,
+                                            n->nhdr + n->len, &nread, &algo);
+  if (err)
+    cert->last_error = err;
   else 
     cert->cache.digest_algo = algo;
 
@@ -456,7 +475,7 @@ ksba_cert_get_digest_algo (ksba_cert_t cert)
  * @cert: certificate object 
  * 
  * This function returnes the serial number of the certificate.  The
- * serial number is an integer returned as an cancnical encoded
+ * serial number is an integer returned as an canonical encoded
  * S-expression with just one element.
  * 
  * Return value: An allocated S-Exp or NULL for no value.
@@ -645,15 +664,16 @@ get_name (ksba_cert_t cert, int idx, int use_subject, char **result)
         }
       else if (ti.tag == 2 || ti.tag == 6)
         { /* dNSName or URI - this are implicit IA5_STRINGs */
-          char numbuf[30];
+          char numbuf[20], *numbufp;
+          size_t numbuflen;
 
-          snprintf (numbuf, DIM(numbuf), "%lu:", ti.length);
-          p = xtrymalloc (11 + strlen (numbuf) + ti.length + 3);
+          numbufp = smklen (numbuf, DIM(numbuf), ti.length, &numbuflen);
+          p = xtrymalloc (11 + numbuflen + ti.length + 3);
           if (!p)
             return gpg_error (GPG_ERR_ENOMEM);
           *result = p;
           p = stpcpy (p, ti.tag == 2? "(8:dns-name" : "(3:uri");
-          p = stpcpy (p, numbuf);
+          p = stpcpy (p, numbufp);
           memcpy (p, der, ti.length);
           p += ti.length;
           *p++ = ')';
@@ -1127,7 +1147,8 @@ ksba_cert_get_key_usage (ksba_cert_t cert, unsigned int *r_flags)
       if (!strcmp (oid, oidstr_keyUsage))
         break;
     }
-  if (gpg_err_code (err) == GPG_ERR_EOF)
+  if (gpg_err_code (err) == GPG_ERR_EOF
+      || gpg_err_code (err) == GPG_ERR_NO_VALUE)
       return gpg_error (GPG_ERR_NO_DATA); /* no key usage */
   if (err)
     return err;
@@ -1355,7 +1376,7 @@ ksba_cert_get_cert_policies (ksba_cert_t cert, char **r_policies)
 
   if (gpg_err_code (err) == GPG_ERR_EOF)
     err = 0;
-  if (!*r_policies)
+  if (!*r_policies || gpg_err_code (err) == GPG_ERR_NO_VALUE)
     err = gpg_error (GPG_ERR_NO_DATA);
 
  leave:
@@ -1446,7 +1467,7 @@ ksba_cert_get_ext_key_usages (ksba_cert_t cert, char **result)
 
   if (gpg_err_code (err) == GPG_ERR_EOF)
     err = 0;
-  if (!*result)
+  if (!*result || gpg_err_code (err) == GPG_ERR_NO_VALUE)
     err = gpg_error (GPG_ERR_NO_DATA);
 
  leave:
@@ -1750,7 +1771,8 @@ ksba_cert_get_auth_key_id (ksba_cert_t cert,
       if (!strcmp (oid, oidstr_authorityKeyIdentifier))
         break;
     }
-  if (gpg_err_code (err) == GPG_ERR_EOF)
+  if (gpg_err_code (err) == GPG_ERR_EOF
+      || gpg_err_code (err) == GPG_ERR_NO_VALUE)
     return gpg_error (GPG_ERR_NO_DATA); /* not available */
   if (err)
     return err;
@@ -1893,7 +1915,8 @@ get_simple_octet_string_ext (ksba_cert_t cert, const char *oid,
     }
   if (err)
     {
-      if (gpg_err_code (err) == GPG_ERR_EOF)
+      if (gpg_err_code (err) == GPG_ERR_EOF
+          || gpg_err_code (err) == GPG_ERR_NO_VALUE)
         return gpg_error (GPG_ERR_NO_DATA);
       return err;
     }
