@@ -1,11 +1,11 @@
 /* ksba.h - X509 library for the Aegypten project
- *      Copyright (C) 2001, 2002, 2003, 2004, 2005 g10 Code GmbH
+ *      Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 g10 Code GmbH
  *
  * This file is part of KSBA.
  *
  * KSBA is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * KSBA is distributed in the hope that it will be useful,
@@ -14,9 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef KSBA_H
@@ -78,7 +76,7 @@ typedef enum
     KSBA_SR_DETACHED_DATA = 8,
     KSBA_SR_BEGIN_ITEMS = 9,
     KSBA_SR_GOT_ITEM = 10,
-    KSBA_SR_END_ITEMS = 11,
+    KSBA_SR_END_ITEMS = 11
   } 
 ksba_stop_reason_t;
 typedef ksba_stop_reason_t KsbaStopReason _KSBA_DEPRECATED;
@@ -390,8 +388,8 @@ gpg_error_t ksba_ocsp_hash_response (ksba_ocsp_t ocsp,
 ksba_sexp_t ksba_ocsp_get_sig_val (ksba_ocsp_t ocsp,
                                    ksba_isotime_t produced_at);
 gpg_error_t ksba_ocsp_get_responder_id (ksba_ocsp_t ocsp,
-                                        ksba_name_t *name,
-                                        unsigned char *sha1keyhash);
+                                        char **r_name,
+                                        ksba_sexp_t *r_keyid);
 ksba_cert_t ksba_ocsp_get_cert (ksba_ocsp_t ocsp, int idx);
 gpg_error_t ksba_ocsp_get_status (ksba_ocsp_t ocsp, ksba_cert_t cert,
                                   ksba_status_t *r_status,
@@ -399,7 +397,11 @@ gpg_error_t ksba_ocsp_get_status (ksba_ocsp_t ocsp, ksba_cert_t cert,
                                   ksba_isotime_t r_next_update,
                                   ksba_isotime_t r_revocation_time,
                                   ksba_crl_reason_t *r_reason);
-
+gpg_error_t ksba_ocsp_get_extension (ksba_ocsp_t ocsp, ksba_cert_t cert,
+                                     int idx,
+                                     char const **r_oid, int *r_crit,
+                                     unsigned char const **r_der, 
+                                     size_t *r_derlen);
 
 
 /*-- certreq.c --*/
@@ -481,6 +483,13 @@ char *ksba_oid_to_str (const char *buffer, size_t length);
 gpg_error_t ksba_oid_from_str (const char *string,
                                unsigned char **rbuf, size_t *rlength);
 
+/*-- dn.c --*/
+gpg_error_t ksba_dn_der2str (const void *der, size_t derlen, char **r_string);
+gpg_error_t ksba_dn_str2der (const char *string,
+                             unsigned char **rder, size_t *rderlen);
+gpg_error_t ksba_dn_teststr (const char *string, int seq, 
+                             size_t *rerroff, size_t *rerrlen);
+
 
 /*-- name.c --*/
 gpg_error_t ksba_name_new (ksba_name_t *r_name);
